
import wolfpack
from wolfpack import additem
from wolfpack.utilities import tobackpack
from wolfpack.consts import *
import random

def onUse( char, item ):
	# Bandages have to be in backpack
	if item.getoutmostchar() != char:
		char.socket.clilocmessage(500295)
		return 1

	# Already Bandaging ??
	if char.socket.hastag( 'using_bandages' ):
		char.socket.sysmessage( 'You are already using bandages.' )
		return 1

	# Display Target
	if item.id == 0xe21 or item.id == 0xee9:
		char.socket.clilocmessage(500948)
		char.socket.attachtarget('bandages.bandage_response', [item.serial])

	elif item.id == 0xe20 or item.id == 0xe22:
		char.socket.sysmessage( 'Where do you want to wash these bandages?' )
		char.socket.attachtarget( 'bandages.wash_response', [ item.serial ] )

	return 1

def wash_response( char, args, target ):
	bandages = wolfpack.finditem( args[0] )

	if not bandages:
		return
		
	if (target.item and target.item.getoutmostchar() and target.item.getoutmostchar() != char) or not char.canreach(target.pos, 5):
		char.socket.sysmessage("You can't reach that.")
		return

	# Did we target something wet?
	id = 0

	if target.item:
		id = target.item.id
	else:
		id = target.model

	# Wash Basins are allowed.
	if not target.item or target.item.id != 0x1008:
		tiledata = wolfpack.tiledata(id)
		if not tiledata[ 'flag1' ] & 0x80:
			char.socket.sysmessage('You cannot wash your bandages here.')
			return

	char.socket.sysmessage('You wash your bandages and put the clean bandages into your backpack.')

	if bandages.id == 0xe20:
		bandages.id = 0xe21
	elif bandages.id == 0xe22:
		bandages.id = 0xee9

	bandages.update()

def validCorpseTarget( char, target ):
	if not target:
		return 0

	if not char.gm and not char.canreach( target, 2 ):
		char.socket.sysmessage( 'You can''t reach that.' )
		return 0

	if target.id != 0x2006:
		char.socket.sysmessage( 'Try using these on a corpse.' )
		return 0

	# Check Owner
	if not target.owner or not target.owner.dead:
		char.socket.sysmessage( 'You can''t help them anymore.' )
		return 0

	return 1

def validCharTarget( char, target ):
	# Do we have a valid target
	if not target:
		char.socket.sysmessage( 'You have to target a living thing.' )
		return 0

	if not char.canreach( target, 2 ):
		char.socket.sysmessage( 'You are too far away to apply bandages on %s' % target.name )
		return 0

	# No resurrection feature in yet
	if target.dead:
		char.socket.sysmessage( 'You can''t heal a ghost.' )
		return 0

	if target.poison != -1:
		char.socket.clilocmessage(1010060)

	# Already at full health
	if target.health >= target.strength:
		if target == char:
			char.socket.sysmessage( 'You are healthy.' )
		else:
			char.socket.sysmessage( '%s does not require you to heal or cure them!' % target.name )
		return 0

	return 1

def bandage_response( char, args, target ):
	corpse = None

	if target.item:
		if validCorpseTarget( char, target.item ):
			corpse = target.item
		else:
			return

	elif target.char and not validCharTarget( char, target.char ):
		return

	elif not target.char:
		char.socket.sysmessage( 'You have to target either a corpse or a creature.' )
		return

	if corpse and ( char.skill[ HEALING ] < 800 or char.skill[ ANATOMY ] < 800 ):
		char.socket.sysmessage( 'You are not skilled enough to resurrect.' )
		return

	# Consume Bandages
	bandages = wolfpack.finditem( args[0] )

	if not bandages:
		return

	baseid = bandages.id
	container = bandages.container

	if bandages.amount == 1:
		bandages.delete()
	else:
		bandages.amount -= 1
		bandages.update()

	success = 0

	# SkillCheck (0% to 80%)
	if not corpse:
		success = char.checkskill( HEALING, 0, 800 )
	else:
		reschance = int( ( char.skill[ HEALING ] + char.skill[ ANATOMY ] ) * 0.17 )
		rescheck = random.randint( 1, 100 )

		if char.checkskill( HEALING, 800, 1000 ) and char.checkskill( ANATOMY, 800, 1000 ) and reschance < rescheck:
			success = 1

	char.action( 0x09 )

	if corpse:
		char.addtimer( random.randint( 2500, 5000 ), 'bandages.bandage_timer', [ 1, success, target.item.serial, baseid ] ) # It takes 5 seconds to bandage
	else:
		if char == target.char:
			char.socket.sysmessage( 'You start applying bandages on yourself' )
		else:
			char.socket.sysmessage( 'You start applying bandages on %s' % target.char.name )
			char.turnto( target.char )

		char.addtimer( random.randint( 1500, 2500 ), 'bandages.bandage_timer', [ 0, success, target.char.serial, baseid ] ) # It takes 5 seconds to bandage

	char.socket.settag( 'using_bandages', 1 )

def bandage_timer( char, args ):
	char.socket.deltag( 'using_bandages' )

	resurrect = args[0]
	success = args[1]
	baseid = args[3]

	# Corpse Target
	if resurrect:
		target = wolfpack.finditem( args[2] )

		owner = target.owner

		if not validCorpseTarget( char, target ):
			return

		if not success:
			char.socket.sysmessage( 'You fail to resurrect the target.' )
			return

		if target.owner:
			target.owner.moveto( target.pos )
			target.owner.update()
			target.owner.resurrect()

			# Move all the belongings from the corpse to the character
			backpack = target.owner.getbackpack()

			for item in target.content:
				# Random Position (for now, maybe storing the original position in a tag would be good)
				# Handle Weight but no Auto Stacking
				backpack.additem( item, 1, 1, 0 )
				item.update()

			target.delete()

			char.socket.sysmessage( 'You successfully resurrect ' + owner.name )
		else:
			char.socket.sysmessage( 'You can''t help them anymore' )

	# Character Target
	else:
		target = wolfpack.findchar( args[2] )

		if not validCharTarget( char, target ):
			return

		if not success:
			if target != char:
				char.socket.sysmessage( 'You fail applying bandages to %s.' % target.name )
			else:
				char.socket.sysmessage( 'You fail applying bandages to yourself.' )
			return

		# Human target ?
		if target.id == 0x190 or target.id == 0x191:
			firstskill = HEALING
			secondskill = ANATOMY
		else:
			firstskill = VETERINARY
			secondskill = ANIMALLORE

		# Heal a bit
		healmin = int( char.skill[ firstskill ] / 5 ) + int( char.skill[ secondskill ] / 5 ) + 3
		healmax = int( char.skill[ firstskill ] / 5 ) + int( char.skill[ secondskill ] / 2 ) + 10

		amount = random.randint( healmin, healmax )

		target.health = min( target.maxhitpoints, target.health + amount )
		target.updatehealth()

		if char == target:
			char.socket.sysmessage( 'You successfully apply bandages on yourself.' )
		else:
			char.socket.sysmessage( 'You successfully apply bandages on %s' % target.name )

	# Create bloody bandages
	# This is target independent
	if baseid == 0xe21:
		item = additem( 'e20' )
		if not tobackpack( item, char ):
			item.update()

	elif baseid == 0xee9:
		item = additem( 'e22' )
		if not tobackpack( item, char ):
			item.update()
